@@ -20,9 +20,9 @@
 # This metadata block gets parsed by setup.py - use single quotes only
 __tagline__ = 'Media aggregator and podcast client'
 __author__ = 'Thomas Perl <thp@gpodder.org>'
-__version__ = '3.10.9'
-__date__ = '2019-06-09'
-__copyright__ = '© 2005-2019 The gPodder Team'
+__version__ = '3.10.13'
+__date__ = '2020-01-29'
+__copyright__ = '© 2005-2020 The gPodder Team'
 __license__ = 'GNU General Public License, version 3 or later'
 __url__ = 'http://gpodder.org/'
 
@@ -82,6 +82,10 @@ except ImportError:
 del sqlite3
 
 
+# Is gpodder running in verbose mode?
+verbose = False
+
+
 # The User-Agent string for downloads
 user_agent = 'gPodder/%s (+%s) %s/%s' % (__version__, __url__, platform.system(), platform.release())
 
