@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2016 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2018 The gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,15 +24,14 @@
 #
 
 
-import gpodder
-from gpodder import util
-from gpodder import jsonconfig
-
 import atexit
+import logging
 import os
 import shutil
 import time
-import logging
+
+import gpodder
+from gpodder import jsonconfig, util
 
 _ = gpodder.gettext
 
@@ -60,25 +59,26 @@ defaults = {
     'limit': {
         'bandwidth': {
             'enabled': False,
-            'kbps': 500.0, # maximum kB/s per download
+            'kbps': 500.0,  # maximum kB/s per download
         },
         'downloads': {
             'enabled': True,
             'concurrent': 1,
+            'concurrent_max': 16,
         },
-        'episodes': 200, # max episodes per feed
+        'episodes': 200,  # max episodes per feed
     },
 
     # Behavior of downloads
     'downloads': {
-        'chronological_order': True, # download older episodes first
+        'chronological_order': True,  # download older episodes first
     },
 
     # Automatic feed updates, download removal and retry on download timeout
     'auto': {
         'update': {
             'enabled': False,
-            'frequency': 20, # minutes
+            'frequency': 20,  # minutes
         },
 
         'cleanup': {
@@ -88,14 +88,14 @@ defaults = {
             'unfinished': True,
         },
 
-        'retries': 3, # number of retries when downloads time out
+        'retries': 3,  # number of retries when downloads time out
     },
 
     # Software updates from gpodder.org
     'software_update': {
-        'check_on_startup': True, # check for updates on start
-        'last_check': 0, # unix timestamp of last update check
-        'interval': 5, # interval (in days) to check for updates
+        'check_on_startup': True,  # check for updates on start
+        'last_check': 0,  # unix timestamp of last update check
+        'interval': 5,  # interval (in days) to check for updates
     },
 
     'ui': {
@@ -114,6 +114,20 @@ defaults = {
 
                     'paned_position': 200,
                     'episode_list_size': 200,
+
+                    'episode_column_sort_id': 0,
+                    'episode_column_sort_order': False,
+                    'episode_column_order': [],
+                },
+                'preferences': {
+                    'width': -1,
+                    'height': -1,
+                    'x': -1, 'y': -1, 'maximized': False,
+                },
+                'config_editor': {
+                    'width': -1,
+                    'height': -1,
+                    'x': -1, 'y': -1, 'maximized': False,
                 },
                 'episode_selector': {
                     'width': 600,
@@ -125,10 +139,15 @@ defaults = {
                     'height': 400,
                     'x': -1, 'y': -1, 'maximized': False,
                 },
+                'export_to_local_folder': {
+                    'width': 500,
+                    'height': 400,
+                    'x': -1, 'y': -1, 'maximized': False,
+                }
             },
 
             'toolbar': False,
-            'new_episodes': 'show', # ignore, show, queue, download
+            'new_episodes': 'show',  # ignore, show, queue, download
             'live_search_delay': 200,
 
             'podcast_list': {
@@ -141,25 +160,27 @@ defaults = {
             'episode_list': {
                 'descriptions': True,
                 'view_mode': 1,
-                'columns': int('110', 2), # bitfield of visible columns
+                'columns': int('110', 2),  # bitfield of visible columns
             },
 
             'download_list': {
                 'remove_finished': True,
             },
+
+            'html_shownotes': True,  # enable webkit renderer
         },
     },
 
     # Synchronization with portable devices (MP3 players, etc..)
     'device_sync': {
-        'device_type': 'none', # Possible values: 'none', 'filesystem', 'ipod'
-        'device_folder': '/media',        
+        'device_type': 'none',  # Possible values: 'none', 'filesystem', 'ipod'
+        'device_folder': '/media',
 
         'one_folder_per_podcast': True,
         'skip_played_episodes': True,
         'delete_played_episodes': False,
 
-        'max_filename_length': 999,
+        'max_filename_length': 120,
 
         'custom_sync_name': '{episode.sortdate}_{episode.title}',
         'custom_sync_name_enabled': False,
@@ -179,13 +200,13 @@ defaults = {
     },
 
     'youtube': {
-        'preferred_fmt_id': 18, # default fmt_id (see fallbacks in youtube.py)
-        'preferred_fmt_ids': [], # for advanced uses (custom fallback sequence)
-        'api_key_v3': '', # API key, register for one at https://developers.google.com/youtube/v3/
+        'preferred_fmt_id': 18,  # default fmt_id (see fallbacks in youtube.py)
+        'preferred_fmt_ids': [],  # for advanced uses (custom fallback sequence)
+        'api_key_v3': '',  # API key, register for one at https://developers.google.com/youtube/v3/
     },
 
     'vimeo': {
-        'fileformat': '720p', # preferred file format (see vimeo.py)
+        'fileformat': '720p',  # preferred file format (see vimeo.py)
     },
 
     'extensions': {
@@ -228,16 +249,17 @@ def config_value_to_string(config_value):
 
     if config_type == list:
         return ','.join(map(config_value_to_string, config_value))
-    elif config_type in (str, unicode):
+    elif config_type in (str, str):
         return config_value
     else:
         return str(config_value)
 
+
 def string_to_config_value(new_value, old_value):
     config_type = type(old_value)
 
     if config_type == list:
-        return filter(None, [x.strip() for x in new_value.split(',')])
+        return [_f for _f in [x.strip() for x in new_value.split(',')] if _f]
     elif config_type == bool:
         return (new_value.strip().lower() in ('1', 'true'))
     else:
@@ -256,6 +278,7 @@ class Config(object):
         self.__observers = []
 
         self.load()
+        self.migrate_defaults()
 
         # If there is no configuration file, we create one here (bug 1511)
         if not os.path.exists(self.__filename):
@@ -321,13 +344,13 @@ class Config(object):
         logger.info('Flushing settings to disk')
 
         try:
-            fp = open(filename+'.tmp', 'wt')
+            fp = open(filename + '.tmp', 'wt')
             fp.write(repr(self.__json_config))
             fp.close()
-            util.atomic_rename(filename+'.tmp', filename)
+            util.atomic_rename(filename + '.tmp', filename)
         except:
             logger.error('Cannot write settings to %s', filename)
-            util.delete_file(filename+'.tmp')
+            util.delete_file(filename + '.tmp')
             raise
 
         self.__save_thread = None
@@ -366,7 +389,7 @@ class Config(object):
         for observer in self.__observers:
             try:
                 observer(name, old_value, value)
-            except Exception, exception:
+            except Exception as exception:
                 logger.error('Error while calling observer %r: %s',
                         observer, exception, exc_info=True)
 
@@ -388,3 +411,9 @@ class Config(object):
 
         setattr(self.__json_config, name, value)
 
+    def migrate_defaults(self):
+        """ change default values in config """
+        if self.device_sync.max_filename_length == 999:
+            logger.debug("setting config.device_sync.max_filename_length=120"
+                         " (999 is bad for NTFS and ext{2-4})")
+            self.device_sync.max_filename_length = 120
