@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2016 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2018 The gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,27 +23,28 @@
 #
 
 
+import logging
+import os
+
 import gpodder
+from gpodder import util, youtube
+
 _ = gpodder.gettext
 
-import logging
 logger = logging.getLogger(__name__)
 
-from gpodder import util
-from gpodder import youtube
-
-import os
 
 class CoverDownloader(object):
     # File name extension dict, lists supported cover art extensions
     # Values: functions that check if some data is of that file type
     SUPPORTED_EXTENSIONS = {
-        '.png': lambda d: d.startswith('\x89PNG\r\n\x1a\n\x00'),
-        '.jpg': lambda d: d.startswith('\xff\xd8'),
-        '.gif': lambda d: d.startswith('GIF89a') or d.startswith('GIF87a'),
+        '.png': lambda d: d.startswith(b'\x89PNG\r\n\x1a\n\x00'),
+        '.jpg': lambda d: d.startswith(b'\xff\xd8'),
+        '.gif': lambda d: d.startswith(b'GIF89a') or d.startswith(b'GIF87a'),
+        '.ico': lambda d: d.startswith(b'\0\0\1\0'),
     }
 
-    EXTENSIONS = SUPPORTED_EXTENSIONS.keys()
+    EXTENSIONS = list(SUPPORTED_EXTENSIONS.keys())
     ALL_EPISODES_ID = ':gpodder:all-episodes:'
 
     # Low timeout to avoid unnecessary hangs of GUIs
@@ -69,7 +70,7 @@ class CoverDownloader(object):
         # If allowed to download files, do so here
         if download:
             # YouTube-specific cover art image resolver
-            youtube_cover_url = youtube.get_real_cover(feed_url)
+            youtube_cover_url = youtube.get_cover(feed_url)
             if youtube_cover_url is not None:
                 cover_url = youtube_cover_url
 
@@ -85,14 +86,14 @@ class CoverDownloader(object):
             try:
                 logger.info('Downloading cover art: %s', cover_url)
                 data = util.urlopen(cover_url, timeout=self.TIMEOUT).read()
-            except Exception, e:
+            except Exception as e:
                 logger.warn('Cover art download failed: %s', e)
                 return self._fallback_filename(title)
 
             try:
                 extension = None
 
-                for filetype, check in self.SUPPORTED_EXTENSIONS.items():
+                for filetype, check in list(self.SUPPORTED_EXTENSIONS.items()):
                     if check(data):
                         extension = filetype
                         break
@@ -107,7 +108,7 @@ class CoverDownloader(object):
                 fp.close()
 
                 return filename + extension
-            except Exception, e:
+            except Exception as e:
                 logger.warn('Cannot save cover art', exc_info=True)
 
         # Fallback to cover art based on the podcast title
@@ -117,5 +118,4 @@ class CoverDownloader(object):
         return os.path.join(gpodder.images_folder, basename)
 
     def _fallback_filename(self, title):
-        return self._default_filename('podcast-%d.png' % (hash(title)%5))
-
+        return self._default_filename('podcast-%d.png' % (hash(title) % 5))
